//modified by Jing Yang after 7/6/2001
#include "ogl.H"
#include "CHierDisplay.H"
#include "XmdvTool.H"
#include "colormap.H"
#include "CFlatParcoord.H"
#include "magnify.H"
#include "brush_oper.H"

/* SID */
#include "db_buffer.h"
#include "db_clock.h"
#include "db_prefetch.h"


CHierDisplay::CHierDisplay(CHierDisplayInformation* hier_info)
{
	this->hier_info= hier_info;
}


CHierDisplay::~CHierDisplay()
{
}


//set this->current_opoacity;
void CHierDisplay::SetCurrentOpacity(double opacity)
{
	current_opacity = opacity;
}


//draw the canvas;
void CHierDisplay::Draw()
{
    if (!dataset) return;
    hierInitClusterTree();

	//set parameters used for drawing, to be rewritten in subclasses;
	SetParameters();
	//set canvas background, clear canvas, and draw axes, to be rewritten in subclasses;
	PrepareCanvas();

	glShadeModel(GL_SMOOTH);

	if ( ! hier_info->database )
	{
		//draw clusters in unbrushed region;
		DrawUnbrushedRegion();
	}
	//draw clusters in brushed region;
	DrawBrushedRegion();

    DrawAxes();
	
    //free space used for drawing, to be rewritten in subclasses;
	FreeSpace();

	if ( hier_info->database )
	{
	}
}


//draw clusters in unbrushed region;
void CHierDisplay::DrawUnbrushedRegion()
{
    //if MAGNIGY, should not draw unbrushed region.
    if ( MAGNIFY ) 
        return;
	
    //if both means and bands of the clusters need not to be shown, return;
	if ( ! ( hier_info->SHOW_MEANS || hier_info->SHOW_BANDS ) )
		return;
	
	//set this->current_opacity;
	double opacity = 1.0;
	if (hier_info->fade_factor>0) 
		opacity=1-hier_info->fade_factor;
	SetCurrentOpacity( opacity );

    bool brushed = false;

   	DrawClusters( brushed );


/*	double left_limit, right_limit, level_of_detail;

	//draw the unbrushed region at left side of the brush; 
	left_limit = 0;
	right_limit = tree_brush[0];
	level_of_detail = cluster_radius;     //cluster_radius is the level of detail of the unbrushed region.

	DrawClusters( left_limit, right_limit, level_of_detail, brushed );
	
	//draw the unbrushed region at right side of the brush; 
	left_limit = tree_brush[1];
	right_limit = 1;

	DrawClusters( left_limit, right_limit, level_of_detail, brushed );
*//*
	//if both means and bands of the clusters need not to be shown, return;
	if ( ! ( SHOW_MEANS || SHOW_BANDS ) )
		return;
	
	//set this->current_opacity;
	double opacity = 1.0;
	if (fade_factor>0) 
		opacity=1-hier_info->fade_factor;
	SetCurrentOpacity( opacity );

	double left_limit, right_limit, level_of_detail;
	bool brushed;

	//draw the unbrushed region at left side of the brush; 
	left_limit = 0;
	right_limit = hier_info->tree_brush[0];
	level_of_detail = hier_info->cluster_radius;     //cluster_radius is the level of detail of the unbrushed region.
	brushed = false;

	DrawClusters( left_limit, right_limit, level_of_detail, brushed );
	
	//draw the unbrushed region at right side of the brush; 
	left_limit = hier_info->tree_brush[1];
	right_limit = 1;

	DrawClusters( left_limit, right_limit, level_of_detail, brushed );
*/
}


//draw clusters in unbrushed region;
void CHierDisplay::DrawBrushedRegion()
{
/*	//if both means and bands of the clusters need not to be shown, return;
	if ( ! ( hier_info->SHOW_MEANS || hier_info->SHOW_BANDS ) )
		return;
	
	//set current opacity
	double opacity = 1.0;
    if (hier_info->fade_factor<0) 
		opacity = 1+hier_info->fade_factor;
	SetCurrentOpacity( opacity );

	bool brushed = true;
   	DrawClusters( brushed );
//////
	
    double left_limit, right_limit, level_of_detail;
    //draw the brushed region; 
	left_limit = tree_brush[0];
	right_limit = tree_brush[1];
	level_of_detail = brushed_radius;     //brushed_radius is the level of detail of the brushed region.
	brushed = true;

	DrawClusters(left_limit, right_limit, level_of_detail, brushed);*/


	//if both means and bands of the clusters need not to be shown, return;
	if ( ! ( hier_info->SHOW_MEANS || hier_info->SHOW_BANDS ) )
		return;
	
	//set current opacity
	double opacity = 1.0;
    if (hier_info->fade_factor<0) 
		opacity = 1+hier_info->fade_factor;
	SetCurrentOpacity( opacity );

	double left_limit, right_limit, level_of_detail;
	bool brushed;

	//draw the brushed region; 
	left_limit = hier_info->tree_brush[0];
	right_limit = hier_info->tree_brush[1];
	level_of_detail = hier_info->brushed_radius;     //brushed_radius is the level of detail of the brushed region.
	brushed = true;

	DrawClusters(left_limit, right_limit, level_of_detail, brushed);

}


void CHierDisplay::TraceDrawBrushedClusters( Cluster node )
{
    if ( ! BITTEST(node.flags,IS_BRUSHED_ANY) )
        return;

    if ( BITTEST(node.flags,IS_CONTOUR1) )
    {
        if ( BITTEST(node.flags,IS_BRUSHED) )
            DrawSingleCluster1(node, true);
        else 
            return;
    }
    else
    {
        for (int j=0; j<node.num_children; j++) 
            TraceDrawBrushedClusters(btree->nodes[node.child[j]]);
    }
}


void CHierDisplay::TraceDrawUnbrushedClusters( Cluster node )
{
	if ( BrushSemantic == 0 )
	{
		 if ( BITTEST(node.flags,IS_BRUSHED) )
			return;
	}

    if ( BITTEST(node.flags,IS_CONTOUR0) )
	{
        if ( ! BITTEST(node.flags,IS_BRUSHED) )
            DrawSingleCluster1(node, false);
        else 
            return;
	}
    else
    {
        for (int j=0; j<node.num_children; j++) 
            TraceDrawUnbrushedClusters(btree->nodes[node.child[j]]);
    }
}

//Given left limit, right limit and level of detail, draw clusters inside this region.
void CHierDisplay::DrawClusters( double left_limit, double right_limit, double level_of_detail, 
									   bool brushed )
{
    if ( hier_info->database )
    {
		/* SID */
	    Cluster c;
	    c.sx.SetLen(dataset->all_dims);
	    c.mean.SetLen(dataset->all_dims);
    	c.max.SetLen(dataset->all_dims);
	    c.min.SetLen(dataset->all_dims);
    	c.all_mean.SetLen(dataset->all_dims);
	    c.num_children = 0;

        c.entries = 0;
	    prepare( left_limit, right_limit, level_of_detail, NO_P_ENTRIES-1);

	    ENTRY *entry;
	    for (init(); (entry=get_one())!=NULL; )
	    {
		    c.color = (entry->data->X+entry->data->Y)/2.;
            c.sqrt_radius = entry->data->L;
		    for (int i=0; i<TABLE_ND; i++)
            {
			    c.mean[i] = entry->data->D[i];
    			c.max[i] = c.mean[i] + c.sqrt_radius;
			    c.min[i] = c.mean[i] - c.sqrt_radius;
                if ( c.min[i] < dataset->all_dim_min[i] )
                {
                    c.min[i] = dataset->all_dim_min[i];
                }
                if ( c.max[i] > dataset->all_dim_max[i] )
                {
                    c.max[i] = dataset->all_dim_max[i];
                }
            }
		    
            DrawSingleCluster1(c, brushed);
	    }
    	register_change();
    }
    else 
    {
        if ( brushed )
        {
            TraceDrawBrushedClusters( btree->nodes[0] );
        }
        else
        {
            TraceDrawUnbrushedClusters( btree->nodes[0] );
        }

    }
  
}


void CHierDisplay::DrawClusters( bool brushed )
{
    if ( brushed )
    {
        TraceDrawBrushedClusters( btree->nodes[0] );
    }
    else
    {
        TraceDrawUnbrushedClusters( btree->nodes[0] );
    }
}
//jing 
//draw a single cluster
void CHierDisplay::DrawSingleCluster1(Cluster &c, bool brushed)
{
    Cluster mappedNode;
    mappedNode.mean.SetLen(dims);
    mappedNode.max.SetLen(dims);
    mappedNode.min.SetLen(dims);
    mappedNode.sx.SetLen(dims);

    mappedNode.num_children = c.num_children;
    mappedNode.color = c.color;
    mappedNode.flags = c.flags;

    if ( !show_two_axes )
    {
        dataset->MapXData(c.mean, mappedNode.mean);
        dataset->MapXData(c.max, mappedNode.max);
        dataset->MapXData(c.min, mappedNode.min);
        dataset->MapXData(c.sx, mappedNode.sx);
        DrawSingleCluster(mappedNode, brushed);
    }
    else
    {
        ARR<double> mean_max;
        ARR<double> mean_min;
        ARR<double> max_max;
        ARR<double> max_min;
        ARR<double> min_max;
        ARR<double> min_min;

        mean_max.SetLen(dims);
        mean_min.SetLen(dims);
        max_max.SetLen(dims);
        max_min.SetLen(dims);
        min_max.SetLen(dims);
        min_min.SetLen(dims);

        dataset->MapDataWithDimClusterExtends(c.mean, mappedNode.mean, mean_max, mean_min);
        dataset->MapDataWithDimClusterExtends(c.max, mappedNode.max, max_max, max_min);
        dataset->MapDataWithDimClusterExtends(c.min, mappedNode.min, min_max, min_min);
        dataset->MapXData(c.sx, mappedNode.sx);
        DrawSingleClusterWithTwoAxes(mappedNode, mean_max, mean_min, max_max, max_min, min_max, min_min, brushed);
    }
}



/******************** the following functions need to be rewritten in the subclasses *****************/
/*****************************************************************************************************/


//draw a single cluster, to be rewritten by subclasses;
void CHierDisplay::DrawSingleCluster(Cluster &c, bool brushed)
{
}


//draw a single cluster, to be rewritten by subclasses;
void CHierDisplay::DrawSingleClusterWithTwoAxes(Cluster &c, ARR<double> mean_max, ARR<double> mean_min, ARR<double> max_max, 
                                                ARR<double> max_min, ARR<double> min_max, ARR<double> min_min, bool brushed)
{
}


//set parameters used for the drawing; to be rewritten by subclasses;
void CHierDisplay::SetParameters()
{
    dims = dataset->dims;

    if ( dim_reducer != NULL && dim_reducer->status == APPLY && dim_reduction_info->disagree_present == MEANBANDS )
        show_dimcluster_band = true;
    else
        show_dimcluster_band = false;

    if ( dim_reducer != NULL && dim_reducer->status == APPLY && dim_reduction_info->disagree_present == TWOAXES )
        show_two_axes = true;
    else
        show_two_axes = false;

    if ( dim_reducer != NULL && dim_reducer->status == APPLY && dim_reduction_info->disagree_present == WIDEAXES )
        show_wide_axes = true;
    else
        show_wide_axes = false;
}


//free space used for drawing; to be rewritten by subclasses;
void CHierDisplay::FreeSpace()
{
}


void CHierDisplay::DrawAxes()
{
}


/******************** the above functions need to be rewritten in the subclasses *********************/
/*****************************************************************************************************/
